package com.hai.config;

import com.alibaba.fastjson.JSONObject;
import com.hai.common.exception.ErrorCode;
import com.hai.common.exception.ErrorHelp;
import com.hai.common.exception.SysCode;
import com.hai.entity.BsProductConfig;
import com.hai.entity.BsProductDiscount;
import com.hai.service.ApiProductService;
import com.hai.service.BsConfigService;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.*;

/**
 * @author hurui
 * @version 1.0
 * @ClassName ThirdProductConfig
 * @description: 第三方接口配置文件
 * @date 2021/7/5 14:18
 */
@Component
public class ThirdProductConfig {

    @Resource
    private BsConfigService bsConfigService;

    @Resource
    private ApiProductService apiProductService;

    // 产品折扣比例
    public BigDecimal getProductConfigNum(Integer productType , Long companyId) {

        Map<String, Object> listMap = new HashMap<>();
        listMap.put("productType", productType);
        listMap.put("companyId", companyId);

        List<BsProductConfig> productConfigs = apiProductService.getProductConfig(listMap);

        if (productConfigs.size() > 0 ) {
            return productConfigs.get(0).getDiscount();
        }
        return  null;
    }

    // 产品折扣比例
    public BsProductConfig getProductConfig(Integer productType , Long companyId , Long productId) {

        Map<String, Object> listMap = new HashMap<>();
        listMap.put("productType", productType);
        listMap.put("companyId", companyId);
        listMap.put("productId", productId);

        List<BsProductConfig> productConfigs = apiProductService.getProductConfig(listMap);

        if (productConfigs.size() > 0 ) {
            return productConfigs.get(0);
        }
        return  null;
    }

    public JSONObject getThirdPartyByDetail(Integer platformId , Integer productType , Long companyId , Long productId) {
        BsProductConfig productConfig = getProductConfig(productType, companyId , productId);

        if (productConfig == null) {
            throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR,  "暂未配置当前内容");
        }

        JSONObject object = new JSONObject();

        Map<String, Object> listMap = new HashMap<>();
        //1:星巴克 2：肯德基 3：充值产品 4 卡券 5 在线加油
        if (productType == 1 || productType == 2 || productType == 3) {
            listMap.put("productType", "5");
        } else if( productType == 4) {
            listMap.put("productType", "1");
        } else if( productType == 5) {
            listMap.put("productType", "2");
        }
        listMap.put("returnType", 1);
        listMap.put("sourceId", productConfig.getId());
        listMap.put("platformId", platformId);
        if (bsConfigService.getProductDiscountByMap(listMap) != null) {
            object.put("integralDiscount" , bsConfigService.getProductDiscountByMap(listMap).getDiscount());
        }

        object.put("priceDiscount" , productConfig.getDiscount());
        object.put("productPayType" , bsConfigService.getProductPayTypeByMap(listMap));
        object.put("productPlatform" , bsConfigService.getProductPlatformByMap(listMap));

        return object;
    }


}
